@@ -1,4 +1,4 @@
-# $Id: LibXML.pm 642 2006-11-17 18:47:25Z pajas $
+# $Id: LibXML.pm 663 2007-04-16 11:38:42Z pajas $
 
 package XML::LibXML;
 
@@ -20,7 +20,7 @@ use IO::Handle; # for FH reads called as methods
 
 BEGIN {
 
-$VERSION = "1.62"; # VERSION TEMPLATE: DO NOT CHANGE
+$VERSION = "1.63"; # VERSION TEMPLATE: DO NOT CHANGE
 require Exporter;
 require DynaLoader;
 @ISA = qw(DynaLoader Exporter);
@@ -111,6 +111,7 @@ $CloseCB = undef;
 # bootstrapping                                                           #
 #-------------------------------------------------------------------------#
 bootstrap XML::LibXML $VERSION;
+undef &AUTOLOAD;
 
 } # BEGIN
 
@@ -146,7 +147,7 @@ sub new {
     }
 
     $self->{XML_LIBXML_EXT_DTD} = 1;
-
+    $self->{_State_} = 0;
     return $self;
 }
 
@@ -312,6 +313,12 @@ sub line_numbers {
     return $self->{XML_LIBXML_LINENUMBERS};
 }
 
+sub no_network {
+    my $self = shift;
+    $self->{XML_LIBXML_NONET} = shift if scalar @_;
+    return $self->{XML_LIBXML_NONET};
+}
+
 sub load_ext_dtd {
     my $self = shift;
     $self->{XML_LIBXML_EXT_DTD} = shift if scalar @_;
@@ -431,6 +438,32 @@ sub __write {
     }
 }
 
+# currently this is only used in the XInlcude processor
+# but in the future, all parsing functions should turn to
+# the new libxml2 parsing API internally and this will
+# become handy
+sub _parser_options {
+  my ($self,$opts)=@_;
+  $opts = {} unless ref $opts;
+  my $flags = 0;
+  $flags |=     1 if  exists $opts->{recover} ? $opts->{recover} : $self->recover;
+  $flags |=     2 if  exists $opts->{expand_entities} ? $opts->{expand_entities} : $self->expand_entities;
+  $flags |=     4 if  exists $opts->{load_ext_dtd} ? $opts->{load_ext_dtd} : $self->load_ext_dtd;
+  $flags |=     8 if  exists $opts->{complete_attributes} ? $opts->{complete_attributes} : $self->complete_attributes;
+  $flags |=    16 if  exists $opts->{validation} ? $opts->{validation} : $self->validation;
+  $flags |=    32 if  $opts->{suppress_errors};
+  $flags |=    64 if  $opts->{suppress_warnings};
+  $flags |=   128 if  exists $opts->{pedantic_parser} ? $opts->{pedantic_parser} : $self->pedantic_parser;
+  $flags |=   256 if  exists $opts->{no_blanks} ? $opts->{no_blanks} : !$self->keep_blanks();
+  $flags |=  1024 if  exists $opts->{expand_xinclude} ? $opts->{expand_xinclude} : $self->expand_xinclude;
+  $flags |=  2048 if  exists $opts->{no_network} ? $opts->{no_network} : $self->no_network;
+  $flags |=  8192 if  exists $opts->{clean_namespaces} ? $opts->{clean_namespaces} : $self->clean_namespaces;
+  $flags |= 16384 if  $opts->{no_cdata};
+  $flags |= 32768 if  $opts->{no_xinclude_nodes};
+  return ($flags);
+}
+
+
 #-------------------------------------------------------------------------#
 # parsing functions                                                       #
 #-------------------------------------------------------------------------#
@@ -624,12 +657,14 @@ sub parse_balanced_chunk {
 sub processXIncludes {
     my $self = shift;
     my $doc = shift;
+    my $opts = shift;
+    my $options = $self->_parser_options($opts);
     if ( $self->{_State_} != 1 ) {
         $self->_init_callbacks();
     }
     my $rv;
     eval {
-        $rv = $self->_processXIncludes($doc || " ");
+        $rv = $self->_processXIncludes($doc || " ", $options);
     };
     my $err = $@;
     if ( $self->{_State_} != 1 ) {
@@ -647,10 +682,13 @@ sub processXIncludes {
 sub process_xincludes {
     my $self = shift;
     my $doc = shift;
+    my $opts = shift;
+    my $options = $self->_parser_options($opts);
+
     my $rv;
     $self->_init_callbacks();
     eval {
-        $rv = $self->_processXIncludes($doc || " ");
+        $rv = $self->_processXIncludes($doc || " ", $options);
     };
     my $err = $@;
     $self->_cleanup_callbacks();
@@ -667,14 +705,15 @@ sub process_xincludes {
 
 sub _html_options {
   my ($self,$opts)=@_;
-  return (undef,undef) unless ref $opts;
+  $opts = {} unless ref $opts;
+  #  return (undef,undef) unless ref $opts;
   my $flags = 0;
-  $flags |=     1 if $opts->{recover} || $self->recover;
+  $flags |=     1 if exists $opts->{recover} ? $opts->{recover} : $self->recover;
   $flags |=    32 if $opts->{suppress_errors};
   $flags |=    64 if $opts->{suppress_warnings};
-  $flags |=   128 if $opts->{pedantic_parser} || $self->pedantic_parser;
-  $flags |=   256 if $opts->{no_blanks} || !$self->keep_blanks();
-  $flags |=  2048 if $opts->{no_network};
+  $flags |=   128 if exists $opts->{pedantic_parser} ? $opts->{pedantic_parser} : $self->pedantic_parser;
+  $flags |=   256 if exists $opts->{no_blanks} ? $opts->{no_blanks} : !$self->keep_blanks;
+  $flags |=  2048 if exists $opts->{no_network} ? $opts->{no_network} : !$self->no_network;
   return ($opts->{URI},$opts->{encoding},$flags);
 }
 
@@ -911,10 +950,26 @@ sub setOwnerDocument {
     $doc->adoptNode( $self );
 }
 
-sub serialize_c14n {
-    my $self = shift;
-    return $self->toStringC14N( @_ );
+sub toStringC14N {
+    my ($self, $comments, $xpath) = (shift, shift, shift);
+    return $self->_toStringC14N( $comments || 0,
+				 (defined $xpath ? $xpath : undef),
+				 0,
+				 undef );
 }
+sub toStringEC14N {
+    my ($self, $comments, $xpath, $inc_prefix_list) = @_;
+    if (defined($inc_prefix_list) and !UNIVERSAL::isa($inc_prefix_list,'ARRAY')) {
+      croak("toStringEC14N: inclusive_prefix_list must be undefined or ARRAY");
+    }
+    return $self->_toStringC14N( $comments || 0,
+				 (defined $xpath ? $xpath : undef),
+				 1,
+				 (defined $inc_prefix_list ? $inc_prefix_list : undef));
+}
+
+*serialize_c14n = \&toStringC14N;
+*serialize_exc_c14n = \&toStringEC14N;
 
 1;
 
@@ -926,6 +981,12 @@ package XML::LibXML::Document;
 use vars qw(@ISA);
 @ISA = ('XML::LibXML::Node');
 
+sub actualEncoding {
+  my $doc = shift;
+  my $enc = $doc->encoding;
+  return (defined $enc and length $enc) ? $enc : 'UTF-8';
+}
+
 sub setDocumentElement {
     my $doc = shift;
     my $element = shift;
@@ -970,7 +1031,8 @@ sub serialize {
 #-------------------------------------------------------------------------#
 sub process_xinclude {
     my $self = shift;
-    XML::LibXML->new->processXIncludes( $self );
+    my $opts = shift;
+    XML::LibXML->new->processXIncludes( $self, $opts );
 }
 
 sub insertProcessingInstruction {
